#!/usr/bin/env perl
use strict;
use warnings;
use Test::More;

# ============================================
# CPAN Testers Regression Tests
#
# Verify issues reported by CPAN testers stay fixed.
# See: https://api.cpantesters.org/v3/summary/Ancient/0.08?grade=fail
# ============================================

# Placeholder - all CPAN tester issues have been resolved
pass('No outstanding CPAN tester issues');

done_testing;
